package SRM;

require 5.006;

use strict;
use warnings;

use SRM::RequestStatus;
use URI;

require Exporter;
require DynaLoader;

our @ISA = qw(Exporter DynaLoader);

our $VERSION = '0.2';

=pod

=head1 NAME

SRM

=head1 DESCRIPTION

Secure SRM client using the gSOAP and GSI libraries.

For more information on the interface see L<http://sdm.lbl.gov/srm-wg/>.

=head1 METHODS

The methods I<die> if there was a SOAP fault.
The fault string and details are added to the I<die> message.

=over 4

=cut


=pod

=item B<new($endpoint)>

Constructor method for an SRM client, refering to a server.

    my $srm = SRM->new("srm.example.org:8443");

=cut

sub new ($) {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $endpoint = shift;
    my $prefprotocol = shift;
    $self->{endpoint} = $endpoint;
    $endpoint =~ s/httpg/https/;
    $self->{URI} = URI->new($endpoint);
    $self->{prefprotocol} = $prefprotocol;
    bless ($self, $class);

    return $self;
}

=pod

=item B<ping()>

Ping the remote service and returns an integer value with the result.

    $srm->ping();

=cut

sub ping () {
    my $self = shift;

    return _ping($self->{endpoint});
}

=pod

=item B<setPrefProtocol()>

Sets the protocol preferred by the user.

=cut

sub setPrefProtocol () {
    my $self = shift;

    $self->{prefprotocol} = shift;
}

=pod

=item B<getPrefProtocol()>

Returns the protocol preferred by the user.

=cut

sub getPrefProtocol () {
    my $self = shift;

    return $self->{prefprotocol};
}

=pod

=item B<getProtocols()>

Returns the protocols, supported by the SRM server.

    print "Protocols: ", join(", ", $srm->getProtocols()), "\n";

=cut

sub getProtocols () {
    my $self = shift;

    return @{_getProtocols($self->{endpoint})};
}

=pod

=item B<setHOME($HOME)>

Sets the base directory name for put and get operations, if only
relative file names are specified.

=cut

sub setHOME($) {
    my $self = shift;

    $self->{HOME} = shift;
}

=pod

=item B<$HOME = getHOME()>

Returns the base directory.

=cut

sub getHOME () {
    my $self = shift;

    return $self->{HOME};
}

=pod

=item B<$SURL getSURL($file)>

Returns a properly formatted SURL for the given filename.
It uses the SRM endpoint specification and the HOME
directory if specified and the $file is a relative path.

=cut

sub getSURL ($) {
    my ($self, $file) = @_;
    
    return $file if $file =~ /^srm:/;

    my $surl = 'srm://' . $self->{URI}->host_port();
    $surl .= $self->{HOME} . '/' if exists($self->{HOME}) and (substr($file, 0, 1) ne '/');
    $surl .= $file;
    return $surl;
}

# internal subroutine to generate proper SURLs
sub _reformatSURLs ($) {
    my ($self, $SURLs) = @_;

    foreach my $i (0 .. scalar(@$SURLs) - 1) {
        $SURLs->[$i] = $self->getSURL( $SURLs->[$i]);
    }

    return $SURLs;
}

=pod

=item B<$SRM::RequestStatus put(\@src_file_names, [\@sURLs], [\@sizes], [\@wantPermanent], [\@protocols])>

Prepares the put operation for the I<src_file_names>, with the I<sURLs> site
URLs.  One can also specify the I<sizes>; if they should be permanent
(I<wantPermanent>); and the desired I<protocols>.

All parameters are references to arrays.

If the I<sURLs> are not defined, then they are computed using the 
I<HOME> attribute, appending the source file names to it. 

If the I<sizes> are not defined, they will be set to 0.

The default value for I<wantPermanent> parameter is 'yes'.

The default set of protocols is determined by the L<getProtocols()> call.

=cut

sub put ($;$$$$) {
    my $self = shift;
    my ($src_file_names, $SURLs, $sizes, $wantPermanent, $protocols) = @_;

    # let's walk backward on the optional argument list

    # use the given protocols
    unless (defined ($protocols)) {
    	unless (exists $self->{prefprotocol}) {
	        $protocols = _getProtocols($self->{endpoint});
	} else {
		$protocols = $self->{prefprotocol};
	}
    }

    # yes, we want all of them permanent
    unless (defined ($wantPermanent)) {
        $wantPermanent = [];
        foreach my $i (0 .. scalar(@$src_file_names) - 1) {
            $wantPermanent->[$i] = 1;
        }
    }

    # probably some more sophisticated stat-ing would be better
    unless (defined ($sizes)) {
        $sizes = [];
        foreach my $i (0 .. scalar(@$src_file_names) - 1) {
            $sizes->[$i] = 0;
        }
    }

    # if HOME is set, then we use it
    unless (defined ($SURLs)) {
        $SURLs = [@$src_file_names];
    }

    # we reformat SURLs to make sure they start with srm://
    $self->_reformatSURLs($SURLs);

    my $status = _put($self->{endpoint}, 
        $src_file_names, $SURLs, $sizes, $wantPermanent, $protocols);

    return SRM::RequestStatus::_new($status, $self);
}

=pod

=item B<$SRM::RequestStatus copy(\@src_file_names, [\@sURLs], [\@wantPermanent])>

Prepares the copy operation for the I<src_file_names>, with the I<sURLs> site
URLs.  One can also specify if they should be permanent
(I<wantPermanent>).

All parameters are references to arrays.

If the I<sURLs> are not defined, then they are computed using the 
I<HOME> attribute, appending the source file names to it. 

The default value for I<wantPermanent> parameter is 'yes'.

=cut

sub copy ($;$$) {
    my $self = shift;
    my ($src_file_names, $SURLs, $wantPermanent) = @_;

    # let's walk backward on the optional argument list

    # yes, we want all of them permanent
    unless (defined ($wantPermanent)) {
        $wantPermanent = [];
        foreach my $i (0 .. scalar(@$src_file_names) - 1) {
            $wantPermanent->[$i] = 1;
        }
    }

    # if HOME is set, then we use it
    unless (defined ($SURLs)) {
        $SURLs = [@$src_file_names];
    }

    # we reformat SURLs to make sure they start with srm://
    $self->_reformatSURLs($SURLs);

    my $status = _copy($self->{endpoint}, 
        $src_file_names, $SURLs, $wantPermanent);

    return SRM::RequestStatus::_new($status, $self);
}

=pod

=item B<$SRM::RequestStatus get(\@sURLs, [\@protocols])>

Prepares the get operation for the I<SURLs>.

The default set of protocols is determined by the L<getProtocols()> call.

=cut

sub get ($;$) {
    my $self = shift;
    my ($SURLs, $protocols) = @_;

    # use the given protocols
    unless (defined ($protocols)) {
    	unless (exists $self->{prefprotocol}) {
	        $protocols = _getProtocols($self->{endpoint});
	} else {
		$protocols = $self->{prefprotocol};
	}
    }

    $self->_reformatSURLs ($SURLs);

    my $status = _get($self->{endpoint}, $SURLs, $protocols);

    return SRM::RequestStatus::_new($status, $self);
}

=pod

=item B<$SRM::RequestStatus mkPermanent(\@sURLs)>

Makes the I<SURLs> permanent on the storage element.

=cut

sub mkPermanent ($) {
    my $self = shift;
    my ($SURLs) = @_;

    $self->_reformatSURLs ($SURLs);

    my $status = _mkPermanent($self->{endpoint}, $SURLs);

    return SRM::RequestStatus::_new($status, $self);
}

=pod

=item B<$SRM::RequestStatus pin(\@TURLs)>

Pins the I<TURLs> in the storage element, not allowing them
to leave the disk cache to a secondary storage for fast access.

=cut

sub pin ($) {
    my $self = shift;
    my ($TURLs) = @_;

    my $status = _pin($self->{endpoint}, $TURLs);

    return SRM::RequestStatus::_new($status, $self);
}

=pod

=item B<$SRM::RequestStatus unPin(\@TURLs, $requestId)>

UnPins previously pinned I<TURLs>. One must supply the I<$requestId>
of the previous I<pin> request.

=cut

sub unPin ($$) {
    my $self = shift;
    my ($TURLs, $requestId) = @_;

    my $status = _unPin($self->{endpoint}, $TURLs, $requestId);

    return SRM::RequestStatus::_new($status, $self);
}

=pod

=item B<$SRM::RequestStatus getRequestStatus(id)>

Retrieves an existing request's status information.

        my $status = $srm->get("a");
            $status = $srm->getRequestStatus($status->requestId());

=cut

sub getRequestStatus ($) {
    my $self = shift;
    my ($requestId) = @_;

    my $status = _getRequestStatus($self->{endpoint}, $requestId);

    return SRM::RequestStatus::_new($status, $self);
}

=pod

=item B<$fileMetaData getFileMetaData(\@SURLs)>

Returns file metadata as an array hash references. 
There is no request associated with this call.

=cut

sub getFileMetaData ($) {
    my $self = shift;
    my ($SURLs) = @_;

    $self->_reformatSURLs ($SURLs);

    return _getFileMetaData($self->{endpoint}, $SURLs);
}

=pod

=item B<$SRM::RequestStatus getEstPutTime(\@src_file_names, [\@sURLs], [\@sizes], [\@wantPermanent], [\@protocols])>

Like L<put()>, but does nothing except returns status.

=cut

sub getEstPutTime ($;$$$$) {
    my $self = shift;
    my ($src_file_names, $SURLs, $sizes, $wantPermanent, $protocols) = @_;

    # let's walk backward on the optional argument list

    # use the given protocols
    unless (defined ($protocols)) {
    	unless (exists $self->{prefprotocol}) {
	        $protocols = _getProtocols($self->{endpoint});
	} else {
		$protocols = $self->{prefprotocol};
	}
    }

    # yes, we want all of them permanent
    unless (defined ($wantPermanent)) {
        $wantPermanent = [];
        foreach my $i (0 .. scalar(@$src_file_names) - 1) {
            $wantPermanent->[$i] = 1;
        }
    }

    # probably some more sophisticated stat-ing would be better
    unless (defined ($sizes)) {
        $sizes = [];
        foreach my $i (0 .. scalar(@$src_file_names) - 1) {
            $sizes->[$i] = 0;
        }
    }

    # if HOME is set, then we use it
    unless (defined ($SURLs)) {
        $SURLs = [@$src_file_names];
    }
    # we reformat SURLs to make sure they start with srm://
    $self->_reformatSURLs($SURLs);

    my $status = _getEstPutTime($self->{endpoint}, 
        $src_file_names, $SURLs, $sizes, $wantPermanent, $protocols);

    return SRM::RequestStatus::_new($status, $self);
}

=pod

=item B<$SRM::RequestStatus getEstGetTime(\@sURLs, [\@protocols])>


Like L<get()>, but does nothing except returns status.

=cut

sub getEstGetTime ($;$) {
    my $self = shift;
    my ($SURLs, $protocols) = @_;

    # use the given protocols
    unless (defined ($protocols)) {
    	unless (exists $self->{prefprotocol}) {
	        $protocols = _getProtocols($self->{endpoint});
	} else {
		$protocols = $self->{prefprotocol};
	}
    }

    $self->_reformatSURLs ($SURLs);

    my $status = _getEstGetTime($self->{endpoint}, $SURLs, $protocols);

    return SRM::RequestStatus::_new($status, $self);
}

=pod

=item B<$SRM::RequestStatus setFileStatus($requestId, $fileId, $status)>

Update a file from I<Ready>, or from anything to I<Failed> by the client.

=cut

sub setFileStatus ($;$) {
    my $self = shift;
    my ($requestId, $fileId, $nstatus) = @_;

    my $status = _setFileStatus($self->{endpoint}, $requestId, $fileId, $nstatus);

    return SRM::RequestStatus::_new($status, $self);
}

=pod

=item B<advisoryDelete(\@SURLs)>

Recommends file(s) to be removed before any others. An SRM
implementation may ignore this request.

This call does not return any status information.

=cut

sub advisoryDelete ($) {
    my $self = shift;
    my ($SURLs) = @_;

    $self->_reformatSURLs($SURLs);

    _advisoryDelete($self->{endpoint}, @$SURLs);
}

=pod

=back

=head1 SEE ALSO

L<SRM::RequestStatus>

=head1 AUTHORS

Akos.Frohner <Akos.Frohner@cern.ch>

Copyright (c) 2004 CERN, on behalf of the EU EGEE project.
For license conditions see LICENSE file or
L<http://www.edg.org/license.html>.

=cut

bootstrap SRM $VERSION;

1;

__END__
