package SRM::RequestStatus;

require 5.006;

use strict;
use warnings;

our $VERSION = '0.1';

=pod

=head1 NAME

SRM::RequestStatus

=head1 DESCRIPTION

The methods operate on I<SRM::RequestStatus> objects, returned by the L<SRM>
package.

=over 4

=cut



# internal new method: it is not allocating new 
# storage, but blesses the parameter and adds a
# private variable

sub _new ($$) {
    my $self = shift;
    my $srm = shift;
    
    bless ($self, 'SRM::RequestStatus');
    $self->{__srm} = $srm;

    return $self;
}

=pod 

=item B<dump()>

Dumps the request status to the standard output.

=cut

sub dump () {
    my $self = shift;
    print "Request ID:\t", $self->{requestId}, "\n";
    foreach my $key (keys(%$self)) {
        next if $key eq 'requestId';
        next if $key eq 'fileStatuses';
        next if $key =~ /^__/;
        print "\t$key", (' ' x (18 - length($key))),
            (defined($self->{$key}) ? $self->{$key} : "<null>"), "\n";
    }
    print "   File Statuses:\n";
    my ($status, $skey);
    foreach $status (@{$self->{fileStatuses}}) {
        foreach $skey (keys(%$status)) {
            print "\t  $skey", (' ' x (18 - length($skey))),
                (defined($status->{$skey}) ? $status->{$skey} : "<null>"), "\n";
        }
        print "\n";
    }
}

=pod

=item B<$SRM::RequestStatus getRequestStatus()>

Retrieves an existing request's status information.

        my $status = $srm->get("a");
        $status = $status->getRequestStatus();

=cut

sub getRequestStatus () {
    my $self = shift;

    my $status = SRM::_getRequestStatus($self->{__srm}->{endpoint}, $self->{requestId});

    return _new($status, $self->{__srm});
}

=pod

=item B<state()>

Utility function to return the state.

=cut

sub state () {
    my $self = shift;
    
    return $self->{state};
}

=pod

=item B<id()>

Utility function to return the requestId.

=cut

sub id () {
    my $self = shift;
    
    return $self->{requestId};
}

=pod 

=item B<isPending()>

Returns if the request is in pending state.

=cut

sub isPending() {
    my $self = shift;

    return ($self->{state} =~ /pending/i);
}

=pod

=item B<$SRM::RequestStatus waitWhilePending()>

Re-requests the status while it is in pending state.

=cut

sub waitWhilePending () {
    my $status = shift;
    
    while ($status->isPending()) {
        sleep 1;
        $status = $status->getRequestStatus();
    }

    return $status;
}

=pod

=item B<$SRM::RequestStatus setFileStatus($fileIndex, $status)>

Sets the file status for the I<$fileIndex> file (starts with 0)
in the request. Note that it is not the I<fileId>!

See more at L<SRM::setFileStatus()>.

=cut

sub setFileStatus ($$) {
    my $self = shift;
    my ($fileIndex, $nstatus) = @_;

    my $status = SRM::_setFileStatus($self->{__srm}->{endpoint}, 
        $self->{requestId}, $self->{fileStatuses}->[$fileIndex]->{fileId},
        $nstatus);

    return _new($status, $self->{__srm});
}

=pod

=head1 AUTHORS

Akos.Frohner <Akos.Frohner@cern.ch>

Copyright (c) 2004 CERN, on behalf of the EU EGEE project.
For license conditions see LICENSE file or
L<http://www.edg.org/license.html>.

=cut

